#!/bin/bash

if [ $# -lt 5 ]
    then
        echo "The script should be invoked as ./train_icarl.sh <DATA_ROOT> <BACKBONE> <CHEEM_COMPONENT> <CUDA_VISIBLE_DEVICE>"
        exit 1
fi

if [ $# -gt 5 ]
    then
        echo "The script should be invoked as ./train_icarl.sh <DATA_ROOT> <BACKBONE> <CHEEM_COMPONENT> <CUDA_VISIBLE_DEVICE>"
        exit 1
fi

# https://stackoverflow.com/questions/242538/unix-shell-script-find-out-which-directory-the-script-file-resides
# Absolute path to this script
SCRIPT=$(readlink -f "$0")
SCRIPT_DIR=$(dirname "$SCRIPT")
CHEEM_DIR="$SCRIPT_DIR/../.."
TIMM_DIR="$CHEEM_DIR/external/pytorch-image-models"

export PYTHONPATH="$PYTHONPATH:$TIMM_DIR:$CHEEM_DIR"
export CUDA_VISIBLE_DEVICES=$4
export CUDA_LAUNCH_BLOCKING=1

ROOT_DIR="$CHEEM_DIR"
COMMON_TRAIN_CONFIGS="$ROOT_DIR/configs/defaults/common_train_configs.yaml"

# Backbone
BACKBONE=$2
CHEEM_COMPONENT=$3

# Benchmark
BENCHMARK=vdd
NUM_TASKS=10
BENCHMARK_CONFIG_FILE="$ROOT_DIR/configs/$BENCHMARK/${BACKBONE}.yaml"
DATA_ROOT=$1

# ImageNet
IMAGENET_ROOT=$DATA_ROOT
BACKBONE_WEIGHTS_PATH="$CHEEM_DIR/artifacts/backbone_checkpoints/$BACKBONE/imagenet12.pth.tar"

# Experiment settings
UNIT_TEST=''
UNIT_TEST_EPOCHS=2
SEED=42

LOG_WANDB=''
WANDB_PROJECT=cleanup-tests-cheem-$BACKBONE-5
WANDB_USERNAME=user
EXEMPLARS_PER_CLASS=$5

EXP_NAME=icarl-$EXEMPLARS_PER_CLASS-$SEED-$CHEEM_COMPONENT

python3  $SCRIPT_DIR/train_icarl.py --config $COMMON_TRAIN_CONFIGS --root_dir $ROOT_DIR \
    --benchmark $BENCHMARK \
    --runtime-config-file $BENCHMARK_CONFIG_FILE \
    --data_root $DATA_ROOT --imagenet_root $IMAGENET_ROOT \
    --model $BACKBONE \
    --backbone_weights_path $BACKBONE_WEIGHTS_PATH \
    --cheem_component $CHEEM_COMPONENT \
    --amp \
    --dev-percent 0.0 --task-idx 9 --offset_task_labels \
    --lr 1e-3 --min-lr 1e-6 --warmup-lr 1e-4 --warmup-epochs 5 --cooldown-epochs 0 --epochs 20 --batch-size 512 --repeated-sampling --min-batch-size 128  \
    --unit-test-epochs $UNIT_TEST_EPOCHS $UNIT_TEST $LOG_WANDB --wandb-project $WANDB_PROJECT --wandb-username $WANDB_USERNAME --exp-name $EXP_NAME \
    --seed $SEED --smoothing 0.0 --exemplars_per_class $EXEMPLARS_PER_CLASS
